module.exports = {
    extends: [
        require.resolve("eslint:recommended"),
        require.resolve("plugin:cypress/recommended"),
        require.resolve("plugin:chai-friendly/recommended"),
        require.resolve("plugin:json/recommended")
    ],
    parser: "@babel/eslint-parser",
    parserOptions: {
        ecmaVersion: 2021,
        sourceType: "module",
        allowImportExportEverywhere: true
    },
    plugins: [
        "@typescript-eslint/eslint-plugin",
        "extra-rules",
        "json",
        "cypress",
        "simple-import-sort",
        "import",
        "prettier"
    ],
    rules: {
        "extra-rules/no-commented-out-code": "warn",
        "cypress/no-assigning-return-values": "error",
        "cypress/no-unnecessary-waiting": "off",
        "cypress/assertion-before-screenshot": "warn",
        "cypress/no-force": "warn",
        "cypress/no-async-tests": "error",
        "space-in-parens": ["error", "never"],
        "space-before-function-paren": "off", // cannot work with prettier
        "space-before-blocks": "error",
        "json/*": ["error", { allowComments: true }],
        "linebreak-style": 0,
        "no-unused-vars": "off",
        "@typescript-eslint/no-unused-vars": "error",
        "@typescript-eslint/no-explicit-any": "warn",
        "@typescript-eslint/no-non-null-assertion": "warn",
        "@typescript-eslint/ban-ts-comment": "error",
        "no-use-before-define": ["error", { functions: false }],
        "@typescript-eslint/ban-types": [
            "error",
            {
                types: {
                    Object: {
                        message: [
                            'The `Object` type actually means "any non-nullish value", so it is marginally better than `unknown`.',
                            '- If you want a type meaning "any object", you probably want `Record<string, unknown>` instead.',
                            '- If you want a type meaning "any value", you probably want `unknown` instead.'
                        ].join("\n")
                    },
                    "{}": {
                        message: [
                            '`{}` actually means "any non-nullish value".',
                            '- If you want a type meaning "any object", you probably want `Record<string, unknown>` instead.',
                            '- If you want a type meaning "any value", you probably want `unknown` instead.'
                        ].join("\n")
                    },
                    object: {
                        message: [
                            "The `object` type is currently hard to use ([see this issue](https://github.com/microsoft/TypeScript/issues/21732)).",
                            "Consider using `Record<string, unknown>` instead, as it allows you to more easily inspect and use the keys."
                        ].join("\n")
                    }
                }
            }
        ],
        "no-multiple-empty-lines": ["error", { max: 2, maxEOF: 0 }],
        "generator-star-spacing": "off",
        "import/first": "error",
        "import/no-duplicates": "error",
        "import/no-mutable-exports": "error",
        "import/no-unresolved": "error",
        'import/newline-after-import': ['error', { count: 1 }],
        "simple-import-sort/imports": "error",
        "simple-import-sort/exports": "error",
        "@typescript-eslint/naming-convention": [
            "warn",
            {
                selector: ["variable"],
                modifiers: ["const", "global"],
                format: ["UPPER_CASE"]
            }
        ],
        "prettier/prettier": [
            "error",
            {
                endOfLine: "auto"
            }
        ]
    }
};
